#ifndef _genr_h_
#define _genr_h_
#include <adevs.h>
#include "job.h"
/*
The genr class produces jobs periodically.  The genr starts
producing jobs when it receives an input on its start port.
If stops producing jobs when it receives an input on its
stop port.  Jobs appear on the out port.
*/
class genr: public atomic 
{
	public:
		genr(stime_t int_arr_time):
		atomic(),int_arr_time(int_arr_time),count(0){}
		void init()
		{
			hold(int_arr_time);
			count = 0;
		}
		void delta_int()
		{
			count = count + 1;
			hold(int_arr_time);
		}
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x)
		{
			port_iterator start_port(x,start);
			if (!start_port.end_of_input())
				hold(int_arr_time);
			port_iterator stop_port(x,stop);
			if (!stop_port.end_of_input())
				passivate();
		}
		void delta_conf(const adevs_bag<PortValue>& x)
		{
			delta_int();
			delta_ext(0.0,x);
		}
		void output_func(adevs_bag<PortValue>& y)
		{
			output(out,new job(count),y);
		}
		void gc_output(adevs_bag<PortValue>& g)
		{
			for (int i = 0; i < g.getSize(); i++)
				delete g[i].value;
		}

		static const port_t start;
		static const port_t stop;
		static const port_t out;

	private:	
		stime_t int_arr_time;
		int count;
};

const port_t genr::stop(0);
const port_t genr::start(1);
const port_t genr::out(2);
  
#endif
