#include "cell.h"
#include <algorithm>
using namespace std;

double cell::dx = 1.0;
double cell::D[3] = { 0.0,0.0,0.0 };
double cell::min_ta = ADEVS_INFINITY;

cell::cell():
atomic(),
output_count(0)
{
}

cell::cell(adevs_vec<3> u0):
atomic(),
output_count(0)
{
	for (unsigned i = 0; i < 3; i++)
	{
		q.setParams(i,D[i]);
		q.init(i,u0[i]);
	}
}

cell::cell(const cell& src):
atomic(src),
q(src.q),
fl(src.fl),
fr(src.fr),
output_count(src.output_count)
{
}

const cell& cell::operator=(const cell& src)
{
	atomic::operator=(src);
	q = src.q;
	fl = src.fl;
	fr = src.fr;
	output_count = src.output_count;
	return *this;
}

void cell::init()
{
	min_ta = min(min_ta,q.ta());
	hold(q.ta());
}

void cell::delta_int()
{
	output_count++;
	q.integ(q.ta());
	min_ta = min(min_ta,q.ta());
	hold(q.ta());
}

void cell::delta_ext(stime_t e, const adevs_bag<PortValue>& x)
{
	for (int i = 0; i < x.getSize(); i++)
	{
		/// Left boundary flux
		if (x.get(i).port == cell_interface::in[0][1][1])
			fl = x.get(i).value;
		/// Right boundary flux
		else
			fr = x.get(i).value;
	}
	adevs_vec<3> du((fl-fr)/dx);

	for (int i = 0; i < 3; i++)
	{
		if (fabs(du[i]) < D[i])
		{
			du[i] = 0.0;
		}
	}

	q.integ(e,*du);
	hold(q.ta());
}

void cell::delta_conf(const adevs_bag<PortValue>& x)
{
	output_count++;
	delta_ext(q.ta(),x);
	min_ta = min(min_ta,q.ta());
}

void cell::output_func(adevs_bag<PortValue>& y)
{
	adevs_vec<3> yy;
	for (int i = 0; i < 3; i++)
	{
		yy[i] = q.out(i);
	}
	output(cell_interface::out,yy,y);
}

adevs_vec<3> cell::getState() const
{
	adevs_vec<3> s;
	for (int i = 0; i < 3; i++)
		s[i] = q.state(i);
	return s;
}

unsigned long cell::getOutputCount() const
{
	return output_count;
}
