#include "transducer.h"
#include <stdio.h>
#include <algorithm>
using namespace std;

const port_t transducer::sample = 0;

transducer::transducer(const transducer& src):
	atomic(src),
	cell_size(src.cell_size),
	display_lines(src.display_lines),
	inputs(src.inputs),
	mSampleTime(src.mSampleTime){}

const transducer& transducer::operator=(const transducer& src)
{
	atomic::operator=(src);
	cell_size = src.cell_size;
	display_lines = src.display_lines;
	inputs = src.inputs;
	mSampleTime = src.mSampleTime;
	return *this;
}
	
void transducer::init() 
{
	inputs = 0;
	hold(mSampleTime);
}

void transducer::delta_int() 
{
	list<char> display_line;
	int cell_count = cell_size.size();
	for (int i = 0; i < cell_count; i++) 
	{
		if (cell_size[i] >= 2) display_line.push_front(0x01);
		else display_line.push_front(0x00);
	}
	display_lines.push_back(display_line);
	hold(mSampleTime);
}

void transducer::delta_ext(stime_t e, const adevs_bag<PortValue>& x) 
{
	port_iterator sample_port(x,sample);
	for (; !sample_port.end_of_input(); sample_port++) 
	{
		cell_data* data = (cell_data*)(*sample_port);
		cell_size[data->getID()] = data->getSize();
	}
	inputs += x.getSize();
}

void transducer::delta_conf(const adevs_bag<PortValue>& x) 
{
	delta_int();
	delta_ext(0.0,x);
}

void transducer::print_report() 
{
	printf("#define data_width %d\n",display_lines.front().size());
	printf("#define data_height %d\n",display_lines.size());
	printf("#define data_x_hot -1\n");
	printf("#define data_y_hot -1\n");
	printf("static char data_bits[] = {\n");
	list<list<char> >::iterator i;
	list<char>::iterator j;
	char data = 0x00;
	int bit = 0;
	for (i = display_lines.begin(); i != display_lines.end(); i++) 
	{
		for (j = (*i).begin(); j != (*i).end(); j++) 
		{
			data |= (*j << bit);
			bit = (bit + 1) % 8;
			if (bit == 0) 
			{
				printf ("0x%x,",data);
				data = 0x00;
			}
		}
		printf("\n");
	}
	printf("0x00};\n");
	printf("\nInputs: %d\n", inputs);
	printf("tL = %f\n",timeLast());
}
